import { Translations, Lang } from "./components/LajiForm";
interface Query {
    [param: string]: any;
}
interface Options {
    failSilently?: boolean;
    [option: string]: any;
}
export interface ApiClientImplementation {
    fetch: (path: string, query: Query, options: any) => Promise<any>;
}
/**
 * ApiClient "interface". Wraps the given apiClient as a singleton object.
 * Given apiClient must implement fetch().
 */
export default class ApiClient {
    apiClient: ApiClientImplementation;
    lang: Lang;
    translations: Translations;
    cache: {
        [path: string]: {
            [cacheKey: string]: Promise<any>;
        };
    };
    on: {
        [path: string]: (() => void)[];
    };
    constructor(apiClient: ApiClientImplementation, lang: Lang | undefined, translations: Translations);
    /**
     * Implementing apiClient must return a promise that passes the raw response as 1st arg.
     * @param path URL for GET.
     * @param query Object, where keys are param names and values are param values.
     * @returns a Promise.
     */
    fetchRaw(path: string, query?: Query, options?: any): Promise<any>;
    fetch<T>(path: string, query?: Query, options?: Options): Promise<T>;
    getCacheKey(query?: Query, options?: Options): string;
    fetchCached<T>(path: string, query?: Query, options?: Options): Promise<T>;
    invalidateCachePath(path: string): void;
    invalidateCachePathQuery(path: string, query: string): void;
    flushCache: () => void;
    onCachePathInvalidation(path: string, callback: () => void): void;
    removeOnCachePathInvalidation(path: string, callback: () => void): void;
    setLang(lang: Lang): void;
}
export {};
