"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const immutability_helper_1 = require("immutability-helper");
const utils_1 = require("../../utils");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
/**
 * Disables object field's value according a regexp rule that is matched against another field's value.
 * uiSchema = { "ui:options": {
 *  disableField: <object field name>, (field to disable if disableDefiner's value doesn't match the regexp)
 *  disableDefiner: <object field name>, (field that the regexp is matched against)
 *  uiSchema: <uiSchema> (uiSchema used for each object).
 * }}
 */
let DependentDisableField = class DependentDisableField extends React.Component {
    static getName() { return "DependentDisableField"; }
    getStateFromProps(props) {
        let { uiSchema, formData } = props;
        let newFormData = formData;
        this.getUiOptions().rules.forEach(rule => {
            const { disableField, disableDefiner, regexp } = rule;
            const { formData } = props;
            const fieldToMatch = (formData && !utils_1.isNullOrUndefined(formData[disableDefiner])) ?
                formData[disableDefiner] : "";
            if (fieldToMatch.match(regexp)) {
                if (!uiSchema[disableField]) {
                    uiSchema = Object.assign(Object.assign({}, uiSchema), { [disableField]: {} });
                }
                uiSchema = immutability_helper_1.default(uiSchema, {
                    [disableField]: {
                        $merge: {
                            "ui:disabled": true,
                            "ui:inlineHelp": rule.inlineHelp
                        }
                    }
                });
                if ("disabledValueToDisplay" in rule) {
                    newFormData = Object.assign(Object.assign({}, formData), { [disableField]: rule.disabledValueToDisplay });
                }
            }
        });
        return { uiSchema, formData: newFormData };
    }
};
DependentDisableField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            rules: PropTypes.arrayOf(PropTypes.shape({
                disableField: PropTypes.string.isRequired,
                disableDefiner: PropTypes.string.isRequired,
                regexp: PropTypes.string.isRequired,
            })).isRequired
        }),
        uiSchema: PropTypes.object
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object.isRequired
};
DependentDisableField = __decorate([
    VirtualSchemaField_1.default
], DependentDisableField);
exports.default = DependentDisableField;
