"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
const injectionPropType = PropTypes.shape({
    fields: PropTypes.arrayOf(PropTypes.string).isRequired,
    target: PropTypes.string.isRequired
});
/**
 * Inject a schema object property to nested schema.
 * uiSchema = { "ui:options": {
 *  "injections": [
 *    {
 *      "fields": [field1, field2...],
 *      "target": fieldName
 *    }
 *    ...
 *  ]
 * }}
 */
let InjectField = class InjectField extends React.Component {
    constructor() {
        super(...arguments);
        this.onChange = (formData) => {
            const options = this.getUiOptions();
            (Array.isArray(options.injections) ? options.injections.slice(0).reverse() : [options.injections]).forEach((injection) => {
                let { fields, target } = injection;
                target = utils_1.toJSONPointer(target);
                formData = fields.reduce((formData, fieldPointer) => {
                    fieldPointer = utils_1.toJSONPointer(fieldPointer);
                    const fieldName = fieldPointer.split("/").pop();
                    formData = utils_1.updateFormDataWithJSONPointer(Object.assign(Object.assign({}, this.props), { formData }), utils_1.parseJSONPointer(formData, `${target}/${fieldName}`), fieldPointer);
                    formData = utils_1.immutableDelete(formData, `${target}/${fieldName}`);
                    return formData;
                }, formData);
            });
            this.props.onChange(formData);
        };
    }
    static getName() { return "InjectField"; }
    getStateFromProps(props) {
        const options = utils_1.getUiOptions(props.uiSchema);
        const { injections } = options;
        let { schema, uiSchema, idSchema, formData, errorSchema } = props;
        (Array.isArray(injections) ? injections : [injections]).forEach((injection) => {
            let { fields, target } = injection;
            target = utils_1.toJSONPointer(target);
            const origSchema = schema;
            fields.forEach((fieldPath) => {
                fieldPath = utils_1.toJSONPointer(fieldPath);
                const fieldName = fieldPath.split("/").pop();
                schema = utils_1.updateSafelyWithJSONPointer(schema, utils_1.parseJSONPointer(origSchema, utils_1.schemaJSONPointer(origSchema, fieldPath)), `${utils_1.schemaJSONPointer(origSchema, target)}/properties/${fieldName}`);
                schema = utils_1.immutableDelete(schema, utils_1.schemaJSONPointer(origSchema, fieldPath));
                uiSchema = utils_1.updateSafelyWithJSONPointer(uiSchema, utils_1.parseJSONPointer(uiSchema, utils_1.uiSchemaJSONPointer(origSchema, fieldPath)), `${utils_1.uiSchemaJSONPointer(origSchema, target)}/${fieldName}`);
                uiSchema = utils_1.immutableDelete(uiSchema, utils_1.uiSchemaJSONPointer(origSchema, fieldPath));
                const [_formData, _idSchema, _errorSchema] = [formData, idSchema, errorSchema].map(prop => utils_1.immutableDelete(utils_1.updateSafelyWithJSONPointer(prop, utils_1.parseJSONPointer(prop, fieldPath), `${target}/${fieldName}`), fieldPath));
                formData = _formData;
                errorSchema = _errorSchema;
                idSchema = _idSchema;
            });
        });
        return { schema, uiSchema, idSchema, formData, errorSchema, onChange: this.onChange };
    }
};
InjectField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            injections: PropTypes.oneOfType([PropTypes.arrayOf(injectionPropType), injectionPropType]).isRequired,
        }).isRequired,
        uiSchema: PropTypes.object
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object.isRequired
};
InjectField = __decorate([
    VirtualSchemaField_1.default
], InjectField);
exports.default = InjectField;
