import * as React from "react";
import { FieldProps } from "../LajiForm";
export declare const colIsLoading: (col: SortCol) => boolean;
interface SortCol {
    name: string;
    /**
     * Undefined if shouldn't sort at all.
     */
    descending?: boolean;
    /**
     * Compare function provided by the compare strategy. It can be asynchronously set.
     * If it returns undefined, comparison will fallback to the default algorithm.
    */
    compare?: (a: any, b: any, sortCol: SortCol, schema: any) => number | undefined;
    /**
     * The chosen compare strategy object.
    */
    compareStrategy: CompareStrategy;
    loading?: boolean;
}
interface State {
    uiSchema?: any;
    sortCols: SortCol[];
}
declare type DefaultCompareStrategy = {
    strategy: "default";
    noDescending?: boolean;
};
declare type TaxonomicCompareStrategy = {
    strategy: "taxonomic";
    noDescending?: boolean;
    valueField?: string;
    query: {
        informalGroupsFilter?: string;
        onlyFinnish?: boolean;
    };
};
declare type CompareStrategy = DefaultCompareStrategy | TaxonomicCompareStrategy;
/**
 * Sorts date according to selected columns. This component provides only the sorting logic,
 * a child component must handle the UI for selecting the sort columns.
 *
 * Can be used only if items are objects (non-objects don't have UUIDs which are used for
 * keeping the original order intact upon changes);
 */
export default class SortArrayField extends React.Component<FieldProps, State> {
    state: State;
    sortTimeIdToSortedIdx: Record<string, number>;
    /**
     * Id-to-idx mapping when the sorting was touched. Used to sorting newly items where they are put, instead of sorting them according to the sort cols.
    */
    sortTimeIdToOrigIdx: Record<string, number>;
    constructor(props: FieldProps);
    componentDidUpdate(): void;
    /**
     * Sort cols might be retrieved from settings JSON, which doesn't hold the 'compare' fn. We add the fn
     * asynchronously when the  component updates according to the comparison strategy name.
     */
    syncColumns(): void;
    getUiShema(props: FieldProps, { sortCols }: State, sortedData: any[]): any;
    getSortableColumns(props: FieldProps): string[] | undefined;
    getNextComponentProps(props: any, state: State): {
        uiSchema: any;
        formData: any[];
        onChange: (formData: any[]) => void;
    };
    updateSortCol(sortCol: SortCol): void;
    setSortCols(sortCols: SortCol[]): void;
    onSortToggle(name: string): void;
    onChange: (formData: any[]) => void;
    render(): JSX.Element;
}
export {};
