"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const utils_2 = require("@rjsf/utils");
const components_1 = require("../components");
const ArrayFieldTemplate_1 = require("../templates/ArrayFieldTemplate");
const BaseComponent_1 = require("../BaseComponent");
const ReactContext_1 = require("../../ReactContext");
const specialRules = {
    legEditors: {
        filterProperties: ({ properties, formData, uiSchemaContext }) => {
            if (!uiSchemaContext)
                return properties;
            const { creator } = uiSchemaContext;
            return (!formData ||
                formData.some(item => item && item.leg && item.leg.match(/MA\.\d+/) && item.leg !== creator)) ? properties : properties.filter(field => field !== "editors");
        }
    }
};
const specialRulesPropTypes = PropTypes.oneOf(["legEditors"]);
let TableField = class TableField extends React.Component {
    render() {
        const { schema, uiSchema = {}, formData, registry: { fields: { ArrayField } }, formContext } = this.props;
        const { uiSchemaContext } = formContext;
        const schemaProps = schema.additionalItems ? schema.additionalItems.properties : schema.items.properties;
        const options = utils_1.getUiOptions(uiSchema);
        let schemaPropsArray = Object.keys(schemaProps);
        const optionsSpecialRules = options.specialRules;
        if (optionsSpecialRules) {
            (Array.isArray(optionsSpecialRules) ? optionsSpecialRules : [optionsSpecialRules]).forEach(specialRule => {
                schemaPropsArray = specialRules[specialRule].filterProperties({
                    properties: schemaPropsArray, uiSchema, uiSchemaContext, formData
                });
            });
        }
        const schemaLength = schemaPropsArray.filter(col => !utils_1.isHidden(uiSchema.items, col)).length;
        const defaultCol = parseInt(12 / schemaLength);
        const defaultWrapperCol = parseInt(12 / (1 + (Object.keys(schemaProps).filter(col => !utils_1.isHidden(uiSchema.items, col)).length - schemaLength)));
        const cols = { xs: undefined, sm: undefined, md: undefined, lg: undefined };
        const wrapperCols = Object.assign({}, cols);
        Object.keys(cols).forEach(col => {
            cols[col] = options[col] ? Math.min(options[col], defaultCol) : defaultCol;
            wrapperCols[col] = options[col] ? Math.min(options[col], defaultWrapperCol) : defaultWrapperCol;
        });
        const itemsSchema = schemaPropsArray.reduce((constructedSchema, prop) => {
            constructedSchema.properties[prop] = schema.items.properties[prop];
            return constructedSchema;
        }, Object.assign(Object.assign({}, schema.items), { properties: {} }));
        const itemsUiSchema = {
            "ui:field": "GridLayoutField",
            "ui:options": Object.assign(Object.assign({}, cols), { label: false }),
        };
        let _uiSchema = uiSchema;
        if (uiSchema.items) {
            if (uiSchema.items.uiSchema) {
                _uiSchema = Object.assign(Object.assign({}, _uiSchema), { items: utils_1.updateTailUiSchema(uiSchema.items, { $merge: { uiSchema: itemsUiSchema } }) });
            }
            else if (uiSchema.items["ui:field"] || uiSchema.items["ui:functions"]) {
                _uiSchema = Object.assign(Object.assign({}, _uiSchema), { items: Object.assign(Object.assign({}, _uiSchema.items), { uiSchema: itemsUiSchema }) });
            }
            else {
                _uiSchema = Object.assign(Object.assign({}, _uiSchema), { items: Object.assign(Object.assign(Object.assign({}, (uiSchema.items || {})), itemsUiSchema), { "ui:options": Object.assign(Object.assign({}, utils_1.getUiOptions(uiSchema.items)), utils_1.getUiOptions(itemsUiSchema)) }) });
            }
        }
        _uiSchema["ui:ArrayFieldTemplate"] = TableArrayFieldTemplate;
        const _formContext = Object.assign(Object.assign({}, formContext), { cols,
            wrapperCols,
            schemaPropsArray });
        return (React.createElement(ArrayField, Object.assign({}, this.props, { schema: Object.assign(Object.assign({}, schema), { items: itemsSchema }), uiSchema: _uiSchema, registry: Object.assign(Object.assign({}, this.props.registry), { formContext: _formContext }), formContext: _formContext })));
    }
};
TableField.contextType = ReactContext_1.default;
TableField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            specialRules: PropTypes.oneOfType([
                specialRulesPropTypes,
                PropTypes.arrayOf(specialRulesPropTypes)
            ])
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array"]),
        items: PropTypes.oneOfType([PropTypes.object, PropTypes.array])
    }).isRequired
};
TableField = __decorate([
    BaseComponent_1.default
], TableField);
exports.default = TableField;
let TableArrayFieldTemplate = class TableArrayFieldTemplate extends React.Component {
    render() {
        const { props } = this;
        const { schema, uiSchema, formContext: { cols, wrapperCols, schemaPropsArray }, idSchema, readonly, disabled } = props;
        const schemaProps = schema.additionalItems ? schema.additionalItems.properties : schema.items.properties;
        const { Label } = this.props.formContext;
        const { Row, Col } = this.context.theme;
        const labels = utils_2.orderProperties(schemaPropsArray.filter(col => !utils_1.isHidden(uiSchema.items, col)), (uiSchema.items || {})["ui:order"]).map(propName => {
            const propUiSchema = uiSchema && uiSchema.items && uiSchema.items[propName] || {};
            return (React.createElement(Col, Object.assign({}, cols, { key: propName + "-label" }),
                React.createElement(Label, { label: "title" in schemaProps[propName] ? schemaProps[propName].title : propName, disabled: false, id: idSchema[propName].$id, required: (schema.items.required && schema.items.required.indexOf(propName) > -1)
                        || propUiSchema["ui:required"], uiSchema: propUiSchema })));
        });
        const options = utils_1.getUiOptions(props.uiSchema);
        const { confirmDelete, deleteCorner, removable = true, nonRemovables = [], buttons, "ui:deleteHelp": deleteHelp } = options;
        if (!this.deleteButtonRefs)
            this.deleteButtonRefs = [];
        const getRefFor = i => elem => { this.deleteButtonRefs[i] = elem; };
        return (React.createElement("div", { className: "table-field" },
            React.createElement(Row, null,
                React.createElement("div", { className: "laji-form-field-template-item" },
                    React.createElement("div", { className: "laji-form-field-template-schema" }, labels),
                    React.createElement("div", { className: "laji-form-field-template-buttons" }))),
            props.items.map((item, i) => {
                const deleteButton = (React.createElement(components_1.DeleteButton, { ref: getRefFor(i), id: `${props.idSchema.$id}_${i}`, disabled: readonly || disabled, onClick: item.onDropIndexClick(item.index), confirm: confirmDelete, corner: deleteCorner, tooltip: deleteHelp, translations: props.formContext.translations }));
                return (React.createElement(Row, { key: utils_1.getUUID(props.formData[item.index]) || item.key },
                    React.createElement(Col, Object.assign({}, wrapperCols),
                        React.createElement("div", { className: "laji-form-field-template-item keep-vertical" },
                            React.createElement("div", { className: "laji-form-field-template-schema" }, item.children),
                            item.hasRemove && !nonRemovables.includes(item.index) && removable && deleteButton))));
            }),
            ArrayFieldTemplate_1.getButtonElems(buttons, props)));
    }
};
TableArrayFieldTemplate.contextType = ReactContext_1.default;
TableArrayFieldTemplate = __decorate([
    ArrayFieldTemplate_1.handlesArrayKeys
], TableArrayFieldTemplate);
