"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const core_1 = require("@rjsf/core");
const ReactContext_1 = require("../../ReactContext");
const utils_1 = require("../../utils");
class _BaseInputTemplate extends React.Component {
    constructor(props) {
        super(props);
        this.getStateFromProps = (props) => {
            return Object.assign({ value: props.value }, utils_1.getUiOptions(props).inputOptions);
        };
        this.onChange = (value) => {
            if (this.props.formatValue)
                value = this.props.formatValue(value);
            const type = this.props.schema.type;
            // Accept only integers
            if (type === "integer") {
                value = value ? value.replace(/[^0-9-]/g, "") : value;
                value = value ? value.replace(/(.+)-/g, "$1") : value;
            }
            // Accept integers or floats
            if (type === "number") {
                value = value ? value.replace(/[^0-9.-]/g, "") : value;
                value = value ? value.replace(/(\..*)\./g, "$1") : value;
                value = value ? value.replace(/(.+)-/g, "$1") : value;
            }
            this.setState({ value }, () => {
                if (!this.focused) {
                    this.props.onChange(value);
                }
                else {
                    if (this.timeout)
                        clearTimeout(this.timeout);
                    this.timeout = this.props.formContext.setTimeout(() => {
                        this.props.onChange(value);
                    }, 1000);
                }
            });
        };
        this.onFocus = (e) => {
            this.focused = true;
            this.props.onFocus && this.props.onFocus(e);
        };
        this.onBlur = (e) => {
            this.focused = false;
            if (this.state.value !== this.props.value) {
                this.props.onChange(this.state.value);
            }
            if (this.timeout)
                clearTimeout(this.timeout);
            this.props.onBlur && this.props.onBlur(e);
        };
        this.state = this.getStateFromProps(props);
    }
    UNSAFE_componentWillReceiveProps(props) {
        this.setState(this.getStateFromProps(props));
    }
    render() {
        const _a = this.props, { formatValue } = _a, // eslint-disable-line @typescript-eslint/no-unused-vars
        props = __rest(_a, ["formatValue"]);
        const options = props.schema.type === "number" || props.schema.type === "integer"
            ? Object.assign(Object.assign({}, props.options), { inputType: "text" }) : props.options;
        const { BaseInputTemplate } = core_1.getDefaultRegistry().templates;
        return React.createElement(BaseInputTemplate, Object.assign({}, props, this.state, { onChange: this.onChange, onFocus: this.onFocus, onBlur: this.onBlur, options: options }));
    }
}
exports.default = _BaseInputTemplate;
_BaseInputTemplate.contextType = ReactContext_1.default;
