"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const components_1 = require("../components");
const utils_1 = require("../../utils");
const ReactContext_1 = require("../../ReactContext");
class ErrorListTemplate extends React.Component {
    constructor(props) {
        super(props);
        this.expand = () => {
            if (!this.state.popped)
                this.setState({ popped: true });
            this.refs.errorPanel.expand();
            this.refs.warningPanel.expand();
        };
        this.poppedToggle = (e) => {
            e.stopPropagation();
            this.setState({ popped: !this.state.popped, poppedTouched: true });
        };
        this.revalidate = () => {
            this.props.formContext.services.rootInstance.validate();
            this.refs.errorPanel.expand();
            this.refs.warningPanel.expand();
        };
        this.submitWithWarnings = () => {
            this.props.formContext.services.rootInstance.submitWithWarnings();
        };
        this.state = { popped: false, poppedTouched: false };
        this.props.formContext.services.rootInstance.setErrorListInstance(this);
    }
    render() {
        const { errorSchema, schema, formContext, uiSchema } = this.props;
        const { "ui:disabled": disabled, "ui:readonly": readonly } = uiSchema;
        const { translations } = formContext;
        const clickHandler = formContext.services.focus.focus;
        function walkErrors(path, id, errorSchema, uiSchema, defaultTitle) {
            const { __errors } = errorSchema, properties = __rest(errorSchema, ["__errors"]);
            const _schema = utils_1.parseJSONPointer(schema, path);
            const title = _schema.title || defaultTitle;
            let { errors, warnings } = (__errors || []).reduce(({ errors, warnings }, _error) => {
                if (_error.includes("[warning]")) {
                    warnings.push({
                        label: title,
                        error: utils_1.formatErrorMessage(_error),
                        id: id
                    });
                }
                else {
                    errors.push({
                        label: title,
                        error: utils_1.formatErrorMessage(_error),
                        id: id
                    });
                }
                return { errors, warnings };
            }, { errors: [], warnings: [] });
            Object.keys(properties).forEach(prop => {
                let _path = path;
                if (prop.match(/^\d+$/))
                    _path = `${_path}/items`;
                else
                    _path = `${_path}/properties/${prop}`;
                const _defaultTitle = uiSchema["ui:multiLanguage"] ? `${title} (${prop})` : prop;
                const childErrors = walkErrors(_path, `${id}_${prop}`, errorSchema[prop], uiSchema[prop] || {}, _defaultTitle);
                errors = [...errors, ...childErrors.errors];
                warnings = [...warnings, ...childErrors.warnings];
            });
            return { errors, warnings };
        }
        const { Glyphicon } = this.context.theme;
        const { errors, warnings } = walkErrors("", "root", errorSchema, uiSchema, "");
        const footer = (errors.length > 0
            ? React.createElement(components_1.Button, { onClick: this.revalidate },
                React.createElement(Glyphicon, { glyph: "refresh" }),
                " ",
                translations.Revalidate)
            : React.createElement(components_1.Button, { onClick: this.submitWithWarnings, variant: "success", disabled: disabled || readonly }, translations.SubmitWithWarnings));
        return (React.createElement("div", { className: `laji-form-error-list${this.state.popped ? " laji-form-popped" : ""}${errors.length === 0 ? " laji-form-warning-list" : ""}`, style: this.state.popped ? { top: (this.props.formContext.topOffset || 0) + 5 } : null },
            React.createElement(components_1.ErrorPanel, { classNames: "error-panel", ref: "errorPanel", errors: errors, title: translations.Errors, clickHandler: clickHandler, showToggle: true, poppedToggle: this.poppedToggle, footer: footer }),
            React.createElement(components_1.ErrorPanel, { classNames: "warning-panel", ref: "warningPanel", errors: warnings, title: translations.Warnings, clickHandler: clickHandler, showToggle: errors.length === 0, poppedToggle: this.poppedToggle, footer: footer })));
    }
}
exports.default = ErrorListTemplate;
ErrorListTemplate.contextType = ReactContext_1.default;
