"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Context_1 = require("../../Context");
const components_1 = require("../components");
const utils_1 = require("../../utils");
class _FieldTemplate extends React.Component {
    constructor(props) {
        super(props);
        this.canFocus = () => {
            const { formContext } = this.props;
            const { uiSchema = {} } = (formContext.formRef.current || { props: {} }).props;
            return uiSchema.autoFocus !== false;
        };
        this.receiveId = (id) => {
            this.setState({ id }, () => {
                const { idToFocus, idToScroll } = Context_1.default(this.props.formContext.contextId);
                if (this.canFocus() && idToFocus === id) {
                    this.props.formContext.utils.focusAndScroll(idToFocus, idToScroll);
                }
            });
        };
        if (utils_1.getUiOptions(props.uiSchema).reserveId === false) {
            this.state = {};
            return;
        }
        const id = this.props.formContext.services.DOMIds.reserve(this.props.id, this.receiveId);
        if (id) {
            this.state = { id };
        }
        else {
            this.state = {};
        }
    }
    componentDidMount() {
        const { formContext } = this.props;
        const contextId = formContext.contextId;
        const _context = Context_1.default(contextId);
        const { idToFocus, idToScroll } = _context;
        if (this.canFocus() && idToFocus !== undefined && this.state.id === idToFocus) {
            if (this.props.formContext.utils.focusAndScroll(idToFocus, idToScroll)) {
                _context.idToFocus = undefined;
                _context.idToScroll = undefined;
            }
        }
    }
    UNSAFE_componentWillReceiveProps(props) {
        if (utils_1.getUiOptions(props.uiSchema).reserveId !== false && this.props.id !== props.id) {
            this.props.formContext.services.DOMIds.release(this.props.id, this.receiveId);
            const id = props.formContext.services.DOMIds.reserve(props.id, this.receiveId);
            id && this.receiveId(id);
        }
    }
    componentWillUnmount() {
        if (utils_1.getUiOptions(this.props.uiSchema).reserveId === false) {
            return;
        }
        this.props.formContext.services.DOMIds.release(this.props.id, this.receiveId);
    }
    render() {
        const { id, classNames: _classNames, children, rawErrors, description, hidden, required, displayLabel, schema, uiSchema, formContext, label: _label, forceDisplayLabel } = this.props;
        const label = "ui:title" in uiSchema
            ? uiSchema["ui:title"]
            : "title" in schema
                ? schema.title
                : _label;
        if (hidden || uiSchema["ui:field"] === "HiddenField" || uiSchema["ui:widget"] === "HiddenWidget") {
            return children;
        }
        const inlineHelp = uiSchema["ui:inlineHelp"];
        const belowHelp = uiSchema["ui:belowHelp"];
        const htmlId = this.state.id ? `_laji-form_${formContext.contextId}_${this.state.id}` : undefined;
        const _displayLabel = forceDisplayLabel ||
            ((schema.items && schema.items.enum && !utils_1.isMultiSelect(schema, uiSchema)) ? false : displayLabel);
        let warnings = [];
        const errors = (rawErrors || []).reduce((arr, err) => {
            if (err.includes("[warning]")) {
                warnings.push(utils_1.formatErrorMessage(err));
            }
            else {
                arr.push(utils_1.formatErrorMessage(err));
            }
            return arr;
        }, []);
        const warningClassName = (warnings.length > 0 && errors.length === 0) && "laji-form-warning-container";
        const { Label, errorsAsPopup } = this.props.formContext;
        const component = (errorsComponent) => (React.createElement("div", { className: utils_1.classNames(_classNames, warningClassName), id: htmlId },
            label && _displayLabel ? React.createElement(Label, { label: label, uiSchema: uiSchema, id: id, required: required || uiSchema["ui:required"], registry: this.props.registry }) : null,
            _displayLabel && description ? description : null,
            React.createElement("div", null,
                inlineHelp ? React.createElement("div", { className: "pull-left" }, children) : children,
                inlineHelp
                    ? (React.createElement("div", { className: "pull-left" },
                        React.createElement(components_1.Help, { help: inlineHelp, id: `${htmlId}-inline-help` }))) : null),
            belowHelp ?
                React.createElement("div", { className: "small text-muted", dangerouslySetInnerHTML: { __html: belowHelp } }) :
                null,
            errorsComponent));
        const errorsComponent = (React.createElement(React.Fragment, null,
            errors.length > 0 ?
                React.createElement("ul", { id: `laji-form-error-container-${id}`, className: "laji-form-error-container" }, errors.map((error, i) => (React.createElement("li", { key: i }, error))))
                : null,
            warnings.length > 0 ?
                React.createElement("ul", { id: `laji-form-warning-container-${id}`, className: "laji-form-warning-container" }, warnings.map((warning, i) => (React.createElement("li", { key: i }, warning))))
                : null));
        if (errorsAsPopup && (rawErrors || []).length) {
            return React.createElement(components_1.TooltipComponent, { placement: "bottom", tooltip: errorsComponent, className: "location-chooser-errors" }, component());
        }
        return component(errorsComponent);
    }
}
exports.default = _FieldTemplate;
