"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var DateTimeWidget_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.YEAR_MATCH = void 0;
const React = require("react");
const react_dom_1 = require("react-dom");
const PropTypes = require("prop-types");
const DateTimePicker = require("react-widgets/lib/DateTimePicker");
const moment = require("moment");
const momentLocalizer = require("react-widgets-moment");
const localizers_1 = require("react-widgets/lib/util/localizers");
const utils_1 = require("../../utils");
const BaseComponent_1 = require("../BaseComponent");
const ReactContext_1 = require("../../ReactContext");
const DATE_TIME_SEPARATOR = ", ";
exports.YEAR_MATCH = /^\d{4}$/;
let DateTimeWidget = DateTimeWidget_1 = class DateTimeWidget extends React.Component {
    constructor(props) {
        super(props);
        this.focus = () => {
            this.dateTimePickerRef.focus();
        };
        this.parse = (value) => {
            if (!value)
                value = "";
            const { allowOnlyYear } = utils_1.getUiOptions(this.props);
            const hasTime = value.includes(DATE_TIME_SEPARATOR) || this.state.time && !this.state.calendar;
            const onlyYear = allowOnlyYear && this.state.calendar && value.match(exports.YEAR_MATCH);
            let momentValue = moment(value, onlyYear ? "YYYY" : this.state.format);
            if (!momentValue.isValid()) {
                return "";
            }
            const isoValue = onlyYear
                ? value
                : hasTime
                    ? momentValue.toISOString()
                    : momentValue.format("YYYY-MM-DD");
            this.timeWritten = hasTime;
            this.onlyYearWritten = onlyYear;
            return onlyYear ? isoValue : moment(isoValue).toDate();
        };
        this.onChange = (value) => {
            this.props.onChange(value, !!"force");
        };
        this.onToggle = p => {
            if (p !== false)
                this.toggle = p; //"time" or "date"
        };
        this.setRef = (elem) => {
            this.dateTimePickerRef = elem;
        };
        this.setContainerRef = (elem) => {
            this.containerRef = elem;
        };
        this.onTextWidgetFocus = () => {
            this.setState({ textInputFocused: true }, this.focus);
        };
        this.onBlur = () => {
            !utils_1.isDescendant(react_dom_1.findDOMNode(this.containerRef), document.activeElement) && this.setState({ textInputFocused: false });
        };
        this.onDateTimePickerChange = (value) => {
            const momentValue = moment(value);
            let formattedValue = momentValue.format("YYYY-MM-DDTHH:mm");
            if (this.onlyYearWritten) {
                formattedValue = value;
            }
            else if (value !== null && !momentValue.isValid()) {
                formattedValue = this.props.value;
            }
            else if ((!this.toggle && !this.timeWritten) ||
                (this.toggle === "date" && (!this.props.value || !this.props.value.includes("T")))) {
                formattedValue = momentValue.format("YYYY-MM-DD");
            }
            this.onChange(!value ? undefined : formattedValue);
            this.toggle = undefined;
            this.timeWritten = false;
            this.onlyYearWritten = false;
        };
        this.getDateWithCurrentTime = (date) => {
            const { value } = this.props;
            const time = value !== undefined && value !== null ? value.split("T")[1] : false;
            return time ? `${date}T${time}` : date;
        };
        this.setToday = () => {
            this.onChange(this.getDateWithCurrentTime(moment().format("YYYY-MM-DD")));
        };
        this.setYesterday = () => {
            this.onChange(this.getDateWithCurrentTime(moment().subtract(1, "d").format("YYYY-MM-DD")));
        };
        this.setSameAsToday = () => {
            const formData = this.props.formContext.services.rootInstance.getFormData();
            const sameOptions = utils_1.getUiOptions(this.props).showButtons.same || {};
            const { path = "/gatheringEvent/dateBegin" } = sameOptions;
            const today = utils_1.parseJSONPointer(formData, path, !!"safely");
            if (today) {
                this.onChange(today);
            }
        };
        momentLocalizer(moment);
    }
    componentDidMount() {
        this.props.formContext.services.focus.addFocusHandler(this.props.id, this.focus);
    }
    componentWillUnmount() {
        this.props.formContext.services.focus.removeFocusHandler(this.props.id, this.focus);
    }
    getStateFromProps(props) {
        const { lang, translations } = props.formContext;
        const formatLang = lang === "sv" ? "fi" : lang === "en" ? "en-gb" : lang;
        let localeFormats = moment().locale(formatLang)._locale._longDateFormat;
        let { allowOnlyYear, dateFormat = "" } = utils_1.getUiOptions(props);
        let timeFormat = "";
        let placeholder = "";
        if (props.calendar) {
            if (!dateFormat)
                dateFormat = localeFormats.L;
            placeholder += translations[dateFormat.toLowerCase()] || dateFormat;
        }
        if (props.time) {
            timeFormat = localeFormats.LT;
            if (placeholder)
                placeholder += DATE_TIME_SEPARATOR;
            placeholder += translations[timeFormat.toLowerCase()] || timeFormat;
        }
        const format = `${dateFormat}${dateFormat ? DATE_TIME_SEPARATOR : ""}${timeFormat}`;
        const splitted = props.value && props.value.split("T");
        const hasTime = props.value && splitted.length > 1 && splitted[1] !== "";
        let inputFormat = allowOnlyYear && props.value && props.value.match(exports.YEAR_MATCH)
            ? "YYYY"
            : hasTime
                ? format
                : dateFormat;
        const state = {
            dateFormat,
            timeFormat,
            format,
            inputFormat,
            placeholder
        };
        ["value", "disabled", "time", "calendar"].forEach(prop => {
            if (prop in props) {
                state[prop] = props[prop];
                if (prop === "calendar") {
                    state["date"] = props[prop];
                }
            }
        });
        return state;
    }
    render() {
        const { value, readonly, disabled } = this.props;
        const { translations, lang } = this.props.formContext;
        const momentValue = moment(value);
        if (!this.state.textInputFocused && value !== null && !momentValue.isValid()) {
            const { TextWidget } = this.props.registry.widgets;
            return React.createElement(TextWidget, Object.assign({}, this.props, { onFocus: this.onTextWidgetFocus }));
        }
        const options = utils_1.getUiOptions(this.props);
        const showTimeList = options.showTimeList !== undefined ? options.showTimeList : true;
        const culture = lang === "en" ? "en-gb" : lang;
        const datePicker = (React.createElement(DateTimePicker, { ref: this.setRef, date: this.state.calendar, time: this.state.time && showTimeList, format: this.state.inputFormat, timeFormat: this.state.timeFormat, placeholder: this.state.placeholder, onToggle: this.onToggle, onChange: this.onDateTimePickerChange, value: value && momentValue.isValid() ? momentValue.toDate() : null, parse: this.parse, disabled: readonly || disabled, culture: culture, messages: {
                calendarButton: translations.ChooseDate,
                timeButton: translations.ChooseTime
            }, onBlur: this.onBlur }));
        const { showButtons } = options;
        const { ButtonGroup } = this.context.theme;
        return showButtons ? (React.createElement("div", { className: "date-widget", ref: this.setContainerRef },
            React.createElement("div", { className: "date-picker" }, datePicker),
            React.createElement(ButtonGroup, null, this.renderButtons(showButtons)))) : React.createElement("div", { className: "date-widget" }, datePicker);
    }
    renderButtons(showButtons) {
        const { readonly, disabled } = this.props;
        const { translations } = this.props.formContext;
        const buttonDefinitions = {
            today: {
                className: "today",
                label: translations.Today,
                onClick: this.setToday
            },
            yesterday: {
                className: "yesterday",
                label: translations.Yesterday,
                onClick: this.setYesterday
            },
            same: {
                className: "same",
                label: translations.Same,
                onClick: this.setSameAsToday
            }
        };
        const options = showButtons === true
            ? { today: true, yesterday: true }
            : showButtons;
        const { Button } = this.context.theme;
        return Object.keys(options)
            .filter(name => options[name])
            .map(name => buttonDefinitions[name])
            .map(({ className, onClick, label }) => (React.createElement(Button, { key: className, className: className, onClick: onClick, disabled: readonly || disabled }, label)));
    }
    formatValue(value, options, props) {
        if (!value)
            return value;
        const { inputFormat: format } = DateTimeWidget_1.prototype.getStateFromProps(Object.assign(Object.assign({}, props), { date: true, time: true, value }));
        return localizers_1.date.format(value, format, props.formContext.lang);
    }
};
DateTimeWidget.contextType = ReactContext_1.default;
DateTimeWidget.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            showButtons: PropTypes.oneOfType([
                PropTypes.bool,
                PropTypes.shape({
                    today: PropTypes.bool,
                    yesterday: PropTypes.bool,
                    same: PropTypes.oneOfType([
                        PropTypes.bool,
                        PropTypes.shape({
                            path: PropTypes.string
                        })
                    ])
                })
            ]),
            showTimeList: PropTypes.bool,
            allowOnlyYear: PropTypes.bool,
            dateFormat: PropTypes.string
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["string"])
    }),
    value: PropTypes.string
};
DateTimeWidget.defaultProps = {
    type: "text",
    readonly: false,
    disabled: false,
    required: false,
    calendar: true,
    time: true
};
DateTimeWidget = DateTimeWidget_1 = __decorate([
    BaseComponent_1.default
], DateTimeWidget);
exports.default = DateTimeWidget;
