/// <reference types="react" />
import { FieldProps, WidgetProps } from "../components/LajiForm";
export declare type Settings = Record<string, string>;
declare type OnSettingsChange = (settings: Record<string, string>, global: boolean) => void;
/**
 * Handles parsing settings from uiSchema. Settings can be either global or form specific (it's up to the client using LajiForm to pass the global or form specific settings).
 */
export default class SettingsService {
    settings: Settings;
    private settingSavers;
    private globalSettingSavers;
    private _onSettingsChange;
    constructor(onSettingsChange: OnSettingsChange, settings?: Settings);
    addSettingSaver: (key: string, fn: () => any, global?: boolean) => void;
    removeSettingSaver: (key: string, global?: boolean) => void;
    setSettings(settings?: Settings): void;
    getSettings(global?: boolean): {};
    onSettingsChange: (global?: boolean) => void;
    /**
     * Should be binded at React Class Component constructor. The component will unbind automatically when unmounted.
     * Hooks into the component React hooks componentDidMount, componentWillUnmount & componentDidUpdate.
     *
     * @param that this of the React component
     * @param props The constructor props
     */
    bind<P extends (FieldProps | WidgetProps)>(that: React.Component<P>, props: P): void;
    private loadStateSettings;
    loadContextSettings<P extends FieldProps | WidgetProps>(props: P, context: any): any;
    private loadSettings;
    updateSettingSaver<P extends (FieldProps | WidgetProps)>(that: React.Component<P>): void;
    private getSettingsKey;
    private getIdSchemaId;
}
export {};
