"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("Array", () => {
    let form;
    describe("copy button", () => {
        const uiSchemaForCopy = (type, filter) => ({
            "ui:options": {
                "buttons": [{
                        fn: "copy", type, filter
                    }]
            }
        });
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm();
        }));
        describe("nested object", () => {
            const schema = {
                type: "array",
                items: {
                    type: "object",
                    properties: {
                        a: { type: "string" },
                        b: { type: "string" },
                        1: {
                            type: "object",
                            properties: {
                                a: { type: "string" },
                                b: { type: "string" },
                                2: {
                                    type: "object",
                                    properties: {
                                        a: { type: "string" },
                                        b: { type: "string" },
                                        default: { type: "string", default: "default" },
                                        3: {
                                            type: "object",
                                            properties: {
                                                a: { type: "string" },
                                                b: { type: "string" },
                                                default: { type: "string", default: "default" },
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            };
            const allButDefaultFilled = [{
                    a: "foo",
                    b: "foo",
                    1: {
                        a: "foo",
                        b: "foo",
                        2: {
                            a: "foo",
                            b: "foo",
                            3: {
                                a: "foo",
                                b: "foo",
                            }
                        }
                    }
                }];
            it("works with whitelist", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ schema, formData: allButDefaultFilled, uiSchema: uiSchemaForCopy("whitelist", ["a", "/1/a", "/1/2/a"]) });
                yield form.$locateButton("", "copy").click();
                expect((yield form.getChangedData())[1]).toEqual({
                    a: "foo",
                    1: {
                        a: "foo",
                        2: {
                            a: "foo",
                            default: "default",
                            3: {
                                default: "default"
                            }
                        }
                    }
                });
            }));
            it("works with blacklist", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ schema, formData: allButDefaultFilled, uiSchema: uiSchemaForCopy("blacklist", ["a", "/1/a", "/1/2/a"]) });
                yield form.$locateButton("", "copy").click();
                expect((yield form.getChangedData())[1]).toEqual({
                    a: null,
                    b: "foo",
                    1: {
                        a: null,
                        b: "foo",
                        2: {
                            a: null,
                            b: "foo",
                            default: "default",
                            3: {
                                a: "foo",
                                b: "foo",
                                default: "default"
                            }
                        }
                    }
                });
            }));
        });
        describe("copying form nonexisting array item", () => {
            const schema = {
                type: "array",
                items: {
                    type: "object",
                    properties: {
                        arr: {
                            type: "array",
                            items: {
                                type: "object",
                                properties: {
                                    a: { type: "string" },
                                    b: { type: "string" },
                                    default: { type: "string", default: "default" }
                                }
                            }
                        }
                    }
                }
            };
            const formData = [{
                    arr: [{
                            a: "foo",
                            b: "foo"
                        }]
                }];
            it("works with whitelist", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ schema, formData, uiSchema: uiSchemaForCopy("whitelist", ["/arr/0/a", "/arr/1/b"]) });
                yield form.$locateButton("", "copy").click();
                expect((yield form.getChangedData())[1]).toEqual({
                    arr: [
                        {
                            a: "foo",
                            default: "default"
                        },
                        {
                            b: null,
                            default: "default"
                        }
                    ]
                });
            }));
            it("works with blacklist", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ schema, formData, uiSchema: uiSchemaForCopy("blacklist", ["/arr/0/a", "/arr/1/b"]) });
                yield form.$locateButton("", "copy").click();
                expect((yield form.getChangedData())[1]).toEqual({
                    arr: [
                        {
                            a: null,
                            b: "foo",
                            default: "default"
                        },
                        {
                            b: null,
                            default: "default"
                        }
                    ]
                });
            }));
        });
    });
    describe("keyboard shortcut", () => {
        let form;
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            const shortcuts = { "alt+i": { fn: "insert" } };
            const props = { schema: { type: "array", items: { type: "string" } }, uiSchema: { "ui:shortcuts": shortcuts } };
            form = yield test_utils_1.createForm(props);
        }));
        afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ formData: [] });
        }));
        describe("insert", () => {
            it("works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.$locateButton("", "add").click();
                expect(yield form.$locate("0").isDisplayed()).toBe(true);
                yield protractor_1.browser.driver.switchTo().activeElement().sendKeys(protractor_1.protractor.Key.chord(protractor_1.protractor.Key.ALT, "i"));
                expect(yield form.$locate("1").isDisplayed()).toBe(true);
            }));
            it("keeps entered value", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.$locateButton("", "add").click();
                yield protractor_1.browser.driver.switchTo().activeElement().sendKeys("test");
                yield protractor_1.browser.driver.switchTo().activeElement().sendKeys(protractor_1.protractor.Key.chord(protractor_1.protractor.Key.ALT, "i"));
                expect(yield form.$locate("1").isDisplayed()).toBe(true);
                expect(yield form.$getInputWidget("0").getAttribute("value")).toBe("test");
            }));
        });
    });
});
