"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("Bird point count (MHL.75)", () => {
    let form;
    let $addUnit;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.75" });
        $addUnit = form.$locateButton("gatherings.0.units", "add");
    }));
    it("adds gathering", () => __awaiter(void 0, void 0, void 0, function* () {
        yield test_utils_1.putForeignMarkerToMap();
        expect(yield test_utils_1.isDisplayed(form.$locate("gatherings.0"))).toBe(true);
    }));
    it("adds unit", () => __awaiter(void 0, void 0, void 0, function* () {
        yield $addUnit.click();
        expect(yield test_utils_1.isDisplayed(form.$locate("gatherings.0.units.0"))).toBe(true);
    }));
    it("unit pair count doesn't lose value when typed and new unit entered by shortcut key", () => __awaiter(void 0, void 0, void 0, function* () {
        const text = "123";
        yield form.$getInputWidget("gatherings.0.units.0.unitFact.pairCountInner").sendKeys(text);
        yield protractor_1.browser.driver.switchTo().activeElement().sendKeys(protractor_1.protractor.Key.chord(protractor_1.protractor.Key.ALT, "u"));
        expect(yield form.$getInputWidget("gatherings.0.units.0.unitFact.pairCountInner").getAttribute("value")).toBe(text);
        // Cleanup
        yield form.$locateButton("gatherings.0.units.0", "delete").click();
        yield form.$locateButton("gatherings.0.units.0", "delete").click();
    }));
    it("when two units and 1st is removed, the autosuggest input value updated to the value of the 1st", () => __awaiter(void 0, void 0, void 0, function* () {
        const taxonAutosuggest1 = form.getTaxonAutosuggestWidget("gatherings.0.units.0.identifications.0.taxonVerbatim");
        const taxonAutosuggest2 = form.getTaxonAutosuggestWidget("gatherings.0.units.1.identifications.0.taxonVerbatim");
        const mockQueue = yield form.createMockResponseQueue("/autocomplete/taxon");
        const peippoMock = yield mockQueue.create();
        const mustarastasMock = yield mockQueue.create();
        yield $addUnit.click();
        yield test_utils_1.updateValue(taxonAutosuggest1.$input, "peippo");
        yield peippoMock.resolve([{
                key: "MX.36237",
                value: "peippo",
            }]);
        yield taxonAutosuggest1.waitForPopoverToHide();
        yield $addUnit.click();
        yield test_utils_1.updateValue(taxonAutosuggest2.$input, "mustarastas");
        yield mustarastasMock.resolve([{
                key: "MX.33106",
                value: "mustarastas",
            }]);
        yield taxonAutosuggest2.waitForPopoverToHide();
        yield mockQueue.remove();
        yield form.$locateButton("gatherings.0.units.0", "delete").click();
        expect(yield taxonAutosuggest1.$input.getAttribute("value")).toBe("mustarastas");
    }));
});
