"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
const properties = require("../properties.json");
describe("Image array", () => {
    let form;
    let imgArrayField;
    const schema = {
        type: "array",
        items: {
            type: "string"
        }
    };
    const uiSchema = {
        "ui:field": "ImageArrayField",
        "ui:options": {
            "deleteConfirmPlacement": "bottom"
        }
    };
    const uiSchemaContext = {
        creator: properties.userId
    };
    describe("", () => {
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm();
            yield form.setState({ schema, uiSchema, uiSchemaContext });
            imgArrayField = form.getImageArrayField("");
        }));
        it("renders correctly", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield imgArrayField.$container.isDisplayed()).toBe(true);
            expect(yield imgArrayField.$dropzone.isDisplayed()).toBe(true);
        }));
        it("adds image", () => __awaiter(void 0, void 0, void 0, function* () {
            const { resolve, remove } = yield form.mockImageUpload("");
            yield resolve();
            expect(yield imgArrayField.$$imgs.count()).toBe(1);
            yield remove();
        }));
        it("open metadata modal on click", () => __awaiter(void 0, void 0, void 0, function* () {
            const { resolve, remove } = yield form.setMockResponse("/images/mock", false);
            yield imgArrayField.$$imgInteractives.first().click();
            yield resolve(test_utils_1.mockImageMetadata);
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(imgArrayField.$modal), 5000, "metadata modal didn't show up");
            expect(yield imgArrayField.$modal.isDisplayed()).toBe(true);
            yield remove();
            yield imgArrayField.$modalClose.click();
        }));
        it("deletes image", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.sleep(1000);
            expect(yield imgArrayField.$$imgs.count()).toBe(1);
            expect(yield imgArrayField.$$imgRemoves.first().isDisplayed()).toBe(true);
            const { resolve, remove } = yield form.setMockResponse("/images/mock", false);
            yield imgArrayField.$$imgRemoves.first().click();
            yield imgArrayField.$imgRemoveConfirmButton("0").click();
            expect(yield imgArrayField.$$imgs.count()).toBe(0);
            yield resolve();
            yield remove();
        }));
    });
    it("Image saves even if the image component unmounts", () => __awaiter(void 0, void 0, void 0, function* () {
        const _schema = {
            type: "array",
            minItems: 1,
            items: {
                type: "object",
                properties: {
                    images: schema
                }
            }
        };
        const _uiSchema = {
            items: {
                images: uiSchema
            }
        };
        yield form.setState({ schema: _schema, uiSchema: _uiSchema, formData: [{}] });
        const { resolve, remove } = yield form.mockImageUpload("0");
        yield form.setState({ uiSchema: Object.assign(Object.assign({}, _uiSchema), { "ui:field": "HiddenField" }) });
        yield resolve();
        yield form.setState({ uiSchema: _uiSchema });
        expect(yield protractor_1.$$(".media-container").count()).toBe(1);
        yield remove();
    }));
});
