"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
const merge = require("deepmerge");
describe("InjectField", () => {
    const schema = {
        type: "object",
        properties: {
            a: {
                type: "object",
                properties: {
                    aa: {
                        type: "object",
                        properties: {
                            aaa: { type: "string" }
                        }
                    },
                    ab: { type: "string" }
                }
            },
            b: {
                type: "object",
                properties: {
                    ba: { type: "string" }
                }
            }
        }
    };
    const uiSchemaBase = {
        a: {
            "classNames": "foobar",
            aa: {
                aaa: {
                    "classNames": "foo"
                }
            }
        },
        b: {
            ba: {
                "classNames": "bar"
            }
        }
    };
    const test = (injections) => {
        let uiSchema = uiSchemaBase;
        for (let injection of injections) {
            const [target, field] = injection;
            uiSchema = merge(uiSchema, {
                "ui:field": "InjectField",
                "ui:options": {
                    injections: [
                        {
                            fields: [field],
                            target
                        }
                    ]
                }
            });
        }
        let form;
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm({ schema, uiSchema });
        }));
        it("displays all fields and their ids are kept intact", () => __awaiter(void 0, void 0, void 0, function* () {
            for (let locator of ["a", "a.aa", "a.ab", "a.ab", "b", "b.ba"]) {
                expect(yield form.$locate(locator).isPresent()).toBe(true, `${locator} not displayed`);
            }
        }));
        it("displays injected field as child of target", () => __awaiter(void 0, void 0, void 0, function* () {
            for (let injection of injections) {
                const [target, field] = injection;
                expect(yield protractor_1.$(test_utils_1.lajiFormLocator(test_utils_1.maybeJSONPointerToLocator(target)))
                    .$(test_utils_1.lajiFormLocator(test_utils_1.maybeJSONPointerToLocator(field)))
                    .isDisplayed()).toBe(true);
            }
        }));
        it("uiSchema is kept for injected fields and target", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$locate("a").getAttribute("className")).toMatch(uiSchemaBase.a.classNames);
            expect(yield form.$locate("a.aa.aaa").getAttribute("className")).toMatch(uiSchemaBase.a.aa.aaa.classNames);
            expect(yield form.$locate("b.ba").getAttribute("className")).toMatch(uiSchemaBase.b.ba.classNames);
        }));
        it("modifying formData works and doesn't spill properties", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(form.$getInputWidget("a.aa.aaa"), "foo");
            yield test_utils_1.updateValue(form.$getInputWidget("a.ab"), "foofoo");
            yield test_utils_1.updateValue(form.$getInputWidget("b.ba"), "bar");
            expect(yield form.$getInputWidget("a.aa.aaa").getAttribute("value")).toBe("foo");
            expect(yield form.$getInputWidget("a.ab").getAttribute("value")).toBe("foofoo");
            expect(yield form.$getInputWidget("b.ba").getAttribute("value")).toBe("bar");
            expect(yield form.getChangedData()).toEqual({
                a: {
                    aa: {
                        aaa: "foo"
                    },
                    ab: "foofoo"
                },
                b: {
                    ba: "bar"
                }
            });
        }));
        it("errors are rendered", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ formData: {
                    a: {
                        aa: {
                            aaa: 1
                        },
                        ab: 1
                    },
                    b: {
                        ba: 1
                    }
                }
            });
            yield form.submit();
            expect(yield form.$$getFieldErrors("a.aa.aaa").count()).toBe(1);
            expect(yield form.$$getFieldErrors("a.ab").count()).toBe(1);
            expect(yield form.$$getFieldErrors("b.ba").count()).toBe(1);
        }));
    };
    describe("simple field, simple target", () => {
        test([["b", "a"]]);
    });
    describe("simple field, simple JSON pointer target", () => {
        test([["/b", "a"]]);
    });
    describe("simple target, JSON pointer field", () => {
        test([["b", "/a/aa/aaa"]]);
    });
    describe("JSON pointer target, JSON Pointer field", () => {
        test([["/a/aa", "/b/ba"]]);
    });
    describe("second injection depending on first injection", () => {
        test([
            ["/a/aa", "/a/ab"],
            ["/b", "/a/aa"]
        ]);
    });
});
