"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
describe("Internal UUIDs", () => {
    let form;
    const schema = {
        type: "object",
        properties: {
            arrayOfObjects: {
                type: "array",
                items: {
                    type: "object",
                    properties: {
                        string: {
                            type: "string"
                        },
                        innerArrayOfObjects: {
                            type: "array",
                            items: {
                                type: "object",
                                properties: {
                                    string: {
                                        type: "string"
                                    }
                                }
                            }
                        }
                    }
                }
            },
            object: {
                type: "object",
                properties: {
                    innerObject: {
                        type: "object",
                        properties: {
                            arrayOfObjects: {
                                type: "array",
                                items: {
                                    type: "object",
                                    properties: {
                                        string: {
                                            type: "string"
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            },
            number: {
                type: "number"
            },
        }
    };
    const formData = {
        arrayOfObjects: [
            {
                string: "foo",
                innerArrayOfObjects: [
                    {
                        string: "foo"
                    }
                ]
            }
        ],
        object: {
            innerObject: {
                arrayOfObjects: [
                    {
                        string: "foo"
                    }
                ]
            }
        },
        number: 2
    };
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm();
        yield form.setState({ schema, formData });
    }));
    it("are generated on initialization", () => __awaiter(void 0, void 0, void 0, function* () {
        const { formData } = yield form.getState();
        yield expect(formData.arrayOfObjects[0]._lajiFormId).not.toBe(undefined);
        yield expect(formData.arrayOfObjects[0].innerArrayOfObjects[0]._lajiFormId).not.toBe(undefined);
        yield expect(formData.object.innerObject.arrayOfObjects[0]._lajiFormId).not.toBe(undefined);
    }));
    it("keeps data intact on id generation", () => __awaiter(void 0, void 0, void 0, function* () {
        const { formData } = yield form.getState();
        yield expect(formData.arrayOfObjects[0].string).toBe("foo");
        yield expect(formData.arrayOfObjects[0].innerArrayOfObjects[0].string).toBe("foo");
        yield expect(formData.object.innerObject.arrayOfObjects[0].string).toBe("foo");
        yield expect(formData.number).toBe(2);
    }));
    it("are removed on submit", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.submit();
        const submitted = yield form.getSubmittedData();
        yield expect(submitted.arrayOfObjects[0]).not.toContain("_lajiFormId");
        yield expect(submitted.arrayOfObjects[0].innerArrayOfObjects[0]).not.toContain("_lajiFormId");
        yield expect(submitted.object.innerObject.arrayOfObjects[0]).not.toContain("_lajiFormId");
    }));
    it("keeps data intact when ids removed", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.submit();
        const submitted = yield form.getSubmittedData();
        yield expect(submitted.arrayOfObjects[0].string).toBe("foo");
        yield expect(submitted.arrayOfObjects[0].innerArrayOfObjects[0].string).toBe("foo");
        yield expect(submitted.object.innerObject.arrayOfObjects[0].string).toBe("foo");
        yield expect(submitted.number).toBe(2);
    }));
    it("are removed on change by default", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.$locateButton("arrayOfObjects", "add").click();
        const changed = yield form.getChangedData();
        yield expect(changed.arrayOfObjects[0]).not.toContain("_lajiFormId");
        yield expect(changed.arrayOfObjects[0].innerArrayOfObjects[0]).not.toContain("_lajiFormId");
        yield expect(changed.object.innerObject.arrayOfObjects[0]).not.toContain("_lajiFormId");
    }));
    it("are not removed on change if optimizeOnChange", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.setState({ optimizeOnChange: true });
        yield form.$locateButton("arrayOfObjects", "add").click();
        const changed = yield form.getChangedData();
        yield expect(changed.arrayOfObjects[0]._lajiFormId).not.toBe(undefined);
        yield form.setState({ optimizeOnChange: false });
    }));
    it("are added to new array items", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.setState({ optimizeOnChange: true, formData: Object.assign({}, formData) });
        yield form.$locateButton("arrayOfObjects", "add").click();
        const changed = yield form.getChangedData();
        yield expect(changed.arrayOfObjects[1]._lajiFormId).not.toBe(undefined);
    }));
    describe("use pre-existing ids", () => {
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({
                formData: Object.assign(Object.assign({}, formData), { arrayOfObjects: [Object.assign(Object.assign({}, formData.arrayOfObjects[0]), { id: "ID" })] })
            });
        }));
        it("instead of _lajiFormId", () => __awaiter(void 0, void 0, void 0, function* () {
            const { formData } = yield form.getState();
            yield expect(formData.arrayOfObjects[0].id).toBe("ID");
            yield expect(formData.arrayOfObjects[0]).not.toContain("_lajiFormId");
        }));
        it("and they are kept intact when ids are removed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.submit();
            const submitted = yield form.getSubmittedData();
            yield expect(submitted.arrayOfObjects[0].id).toBe("ID");
        }));
    });
});
