"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("Line transect (MHL.1)", () => {
    let form;
    let $taxon;
    let unitAutocompleteMock;
    let taxonAutocompleteMock;
    const unitAutocompleteResponse = {
        "key": "llx",
        "value": "llx",
        "payload": {
            "unit": {
                "shortHandText": "llx",
                "identifications": [
                    {
                        "taxon": "Luscinia luscinia"
                    }
                ],
                "informalTaxonGroups": [
                    "MVL.1"
                ],
                "unitFact": {
                    "autocompleteSelectedTaxonID": "MX.32819",
                    "lineTransectObsType": "MY.lineTransectObsTypeSong",
                    "lineTransectRouteFieldType": "MY.lineTransectRouteFieldTypeOuter"
                },
                "pairCount": 1,
                "individualCount": 1
            },
            "interpretedFrom": {
                "taxon": "LL",
                "type": "X"
            }
        }
    };
    const taxonAutocompleteResponse = [
        {
            "key": "MX.32819",
            "value": "Luscinia luscinia",
            "payload": {
                "matchingName": "Luscinia luscinia",
                "informalTaxonGroups": [
                    {
                        "id": "MVL.1",
                        "name": "Linnut"
                    }
                ],
                "scientificName": "Luscinia luscinia",
                "scientificNameAuthorship": "(Linnaeus, 1758)",
                "taxonRankId": "MX.species",
                "matchType": "exactMatches",
                "cursiveName": true,
                "finnish": true,
                "species": true,
                "nameType": "MX.scientificName",
                "vernacularName": "satakieli"
            }
        }
    ];
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.1", localFormData: true });
        $taxon = form.$locate("gatherings.0.units.0.identifications.0.taxon");
        unitAutocompleteMock = yield form.setMockResponse("/autocomplete/unit", false);
        taxonAutocompleteMock = yield form.setMockResponse("/autocomplete/taxon", false);
    }));
    const $shortHandForIdx = (idx, _idx) => protractor_1.$(`#root_gatherings_${idx}_units_${_idx}_shortHandText`);
    const $shorthand = $shortHandForIdx(0, 0);
    it("shorthand unit is shown", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(yield $shorthand.isDisplayed()).toBe(true);
    }));
    it("shorthand unit expands code after typing code", () => __awaiter(void 0, void 0, void 0, function* () {
        function waitUntilShorthandShowsSchema() {
            return protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.presenceOf($taxon), 5000, "Code reading timeout");
        }
        yield $shorthand.sendKeys("llx");
        yield $shorthand.sendKeys(protractor_1.protractor.Key.ENTER);
        yield unitAutocompleteMock.resolve(unitAutocompleteResponse);
        yield waitUntilShorthandShowsSchema();
        expect(yield $taxon.isDisplayed()).toBe(true);
    }));
    it("next unit is automatically added", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(yield $shortHandForIdx(0, 1).isDisplayed()).toBe(true);
    }));
    it("unit formData matches response", () => __awaiter(void 0, void 0, void 0, function* () {
        const formData = yield form.getChangedData();
        expect(test_utils_1.filterUUIDs(formData.gatherings[0].units[0])).toEqual(unitAutocompleteResponse.payload.unit);
        expect(formData.gatherings[0].units[0].identifications_0_taxon).toBe(undefined);
        expect(formData.gatherings[0].units[0].unitFact_autocompleteSelectedTaxonID).toBe(undefined);
        expect(formData.gatherings[0].units[0].unitFact_lineTransectObsType).toBe(undefined);
        expect(formData.gatherings[0].units[0].unitFact_lineTransectRouteFieldType).toBe(undefined);
    }));
    it("next unit is focused", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(yield test_utils_1.getFocusedId()).toBe(yield $shortHandForIdx(0, 1).getAttribute("id"));
    }));
    it("prev unit is focused after keyboard navigation", () => __awaiter(void 0, void 0, void 0, function* () {
        yield protractor_1.browser.driver.switchTo().activeElement().sendKeys(protractor_1.protractor.Key.chord(protractor_1.protractor.Key.ALT, protractor_1.protractor.Key.UP));
        expect(yield test_utils_1.getFocusedId()).toBe(yield form.$getInputWidget("gatherings.0.units.0.identifications.0.taxon").getAttribute("id"));
    }));
    it("next gathering is focused after keyboard navigation", () => __awaiter(void 0, void 0, void 0, function* () {
        yield protractor_1.browser.driver.switchTo().activeElement().sendKeys(protractor_1.protractor.Key.chord(protractor_1.protractor.Key.ALT, protractor_1.protractor.Key.RIGHT));
        yield protractor_1.browser.sleep(200);
        expect(yield test_utils_1.getFocusedId()).toBe(yield $shortHandForIdx(1, 0).getAttribute("id"));
    }));
    it("when prev focused taxon autocomplete finishes, data isn't in FlatField format", () => __awaiter(void 0, void 0, void 0, function* () {
        yield taxonAutocompleteMock.resolve(taxonAutocompleteResponse);
        const formData = yield form.getChangedData();
        expect(test_utils_1.filterUUIDs(formData.gatherings[0].units[0])).toEqual(unitAutocompleteResponse.payload.unit);
        expect(formData.gatherings[0].units[0].identifications_0_taxon).toBe(undefined);
    }));
});
