"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
const $blocker = test_utils_1.lajiFormLocate("gatherings.0.geometry").$(".blocker");
const $imageAddModal = protractor_1.$(".media-add-modal");
const $mobileEditorMap = protractor_1.$(".laji-form.fullscreen .laji-form-map");
describe("Mobile form (MHL.51)", () => {
    describe("without formData", () => {
        let form;
        it("navigate to form", () => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm({ id: "MHL.51" });
        }));
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$form.isPresent()).toBe(true);
        }));
        it("image add modal is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $imageAddModal.isPresent()).toBe(true);
        }));
        it("clicking cancel on image add modal hides it", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $imageAddModal.$(".cancel").click();
            expect(yield $imageAddModal.isPresent()).toBe(false);
        }));
    });
    describe("with formData with geometry", () => {
        let form;
        it("navigate to form", () => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm({ id: "MHL.51", localFormData: "MHL.51-geometry" });
        }));
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$form.isPresent()).toBe(true);
        }));
        it("image add modal is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $imageAddModal.isPresent()).toBe(true);
        }));
        it("map doesn't geolocating blocker since it has geometry", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $blocker.isPresent()).toBe(false);
        }));
        it("map doesn't show mobile editor", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.sleep(100);
            expect(yield $mobileEditorMap.isPresent()).toBe(false);
        }));
    });
    describe("with formData with image", () => {
        let form;
        it("navigate to form", () => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm({ id: "MHL.51", localFormData: "MHL.51-image-geometry" });
        }));
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$form.isPresent()).toBe(true);
        }));
        it("image add modal isn't displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $imageAddModal.isPresent()).toBe(false);
        }));
        it("map doesn't geolocating blocker since it has geometry", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $blocker.isPresent()).toBe(false);
        }));
        it("map doesn't show mobile editor", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.sleep(100);
            expect(yield $mobileEditorMap.isPresent()).toBe(false);
        }));
    });
    describe("with formData without image and edit mode", () => {
        let form;
        it("navigate to form", () => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm({ id: "MHL.51", localFormData: "MHL.51-geometry", isEdit: true });
        }));
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$form.isPresent()).toBe(true);
        }));
        it("image add modal isn't displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $imageAddModal.isPresent()).toBe(false);
        }));
    });
});
