"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
const taxonAutocompleteResponse = [{
        "key": "MX.46587",
        "value": "kettu",
        "payload": {
            "matchingName": "kettu",
            "informalTaxonGroups": [
                {
                    "id": "MVL.2",
                    "name": "Nisäkkäät"
                }
            ],
            "scientificName": "Vulpes vulpes",
            "scientificNameAuthorship": "(Linnaeus, 1758)",
            "taxonRankId": "MX.species",
            "matchType": "exactMatches",
            "cursiveName": true,
            "finnish": true,
            "species": true,
            "nameType": "MX.vernacularName",
            "vernacularName": "kettu"
        }
    }];
describe("NAFI (MHL.6)", () => {
    let form;
    let taxonAutosuggest;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.6" });
    }));
    it("can add unit", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.$locateButton("gatherings.0.units", "add").click();
        expect(yield form.$locate("gatherings.0.units.121").isDisplayed()).toBe(true);
    }));
    describe("selecting species name with mouse", () => {
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            taxonAutosuggest = form.getTaxonAutosuggestWidget("gatherings.0.units.121.identifications.0.taxon");
        }));
        it("selects suggestion value", () => __awaiter(void 0, void 0, void 0, function* () {
            const autocompleteMock = yield form.setMockResponse("/autocomplete/taxon", false);
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield autocompleteMock.resolve(taxonAutocompleteResponse);
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$$suggestions.first().click();
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            yield autocompleteMock.remove();
        }));
        it("and is marked as suggested", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
        }));
        it("and closes suggestion list", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield taxonAutosuggest.$suggestionsContainer.isPresent()).toBe(false);
        }));
    });
    describe("selecting species name with keyboard navigation", () => {
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            taxonAutosuggest = form.getTaxonAutosuggestWidget("gatherings.0.units.122.identifications.0.taxon");
        }));
        it("works", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.$locateButton("gatherings.0.units", "add").click();
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.DOWN);
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.ENTER);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
        }));
        it("and is marked as suggested", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
        }));
        it("and closes suggestion list", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield taxonAutosuggest.$suggestionsContainer.isPresent()).toBe(false);
        }));
    });
});
