"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("SingleItemArrayField", () => {
    const uiSchema = {
        "ui:field": "SingleItemArrayField",
    };
    let form;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm();
    }));
    describe("object", () => {
        const schema = {
            type: "array",
            items: {
                type: "object",
                properties: {
                    a: { type: "string" }
                }
            }
        };
        const formData = [{ a: "foo" }, { a: "bar" }];
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ schema, uiSchema, formData });
        }));
        it("uses first item by default and keeps idSchema intact", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$getInputWidget("0.a").isDisplayed()).toBe(true);
        }));
        describe("respects activeIdx and", () => {
            beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:options": { activeIdx: 1 } }) });
            }));
            it("renders only single item", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield form.$locate("1.a").isDisplayed()).toBe(true);
                expect(yield form.$locate("0.a").isPresent()).toBe(false);
            }));
            it("renders form data", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield form.$getInputWidget("1.a").getAttribute("value")).toBe(formData[1].a);
            }));
            it("onChange", () => __awaiter(void 0, void 0, void 0, function* () {
                yield test_utils_1.updateValue(form.$getInputWidget("1.a"), "test");
                expect(yield form.$getInputWidget("1.a").getAttribute("value")).toBe("test");
            }));
            it("renders item errors", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({
                    formData: [{ a: "a" }, { a: 1 }]
                });
                yield form.submit();
                expect(yield form.$$getFieldErrors("1.a").count()).toBe(1);
            }));
            it("renders array errors", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({
                    schema: Object.assign(Object.assign({}, schema), { minItems: 1 }),
                    formData: []
                });
                yield form.submit();
                expect(yield form.$$getFieldErrors("").count()).toBe(1);
            }));
        });
        it("formats title", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:title": "test %{idx} test" }) });
            expect(yield protractor_1.element(protractor_1.by.cssContainingText("span", "test 1 test")).isDisplayed()).toBe(true);
        }));
        it("respects titleClassName option", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:title": "test", "ui:options": { titleClassName: "title-format-test" } }) });
            expect(yield protractor_1.$(".title-format-test").isDisplayed()).toBe(true);
        }));
    });
    describe("string array with unique items", () => {
        const schema = { type: "array", uniqueItems: true, title: "test", items: { type: "string", enum: ["a"], enumNames: ["a"] } };
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ schema, uiSchema });
        }));
        it("is rendered as select", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$getEnumWidget("").$container.isDisplayed()).toBe(true);
        }));
        it("renders title as label", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield protractor_1.$("label").isDisplayed()).toBe(true);
            expect(yield protractor_1.$("legend").isPresent()).toBe(false);
        }));
    });
});
