import { ElementFinder, ElementArrayFinder } from "protractor";
import { JSONSchema7 } from "json-schema";
export declare const EC: import("protractor").ProtractorExpectedConditions;
export declare const getLocatorForContextId: (contextId: number) => (path: string) => string;
export declare const emptyForm: (params?: string) => Promise<any>;
export declare const navigateToForm: (formID: string, params?: string) => Promise<any>;
export declare const lajiFormLocator: (path: string) => string;
export declare const lajiFormLocate: (str: string) => ElementFinder;
export declare const getFocusedElement: () => import("selenium-webdriver").WebElementPromise;
export declare const getFocusedId: () => import("selenium-webdriver").promise.Promise<string>;
export interface Mock {
    resolve: (response?: any, raw?: boolean) => Promise<void>;
    reject: (response?: any, raw?: boolean) => Promise<void>;
    remove: () => Promise<void>;
}
export interface DateWidgetPO {
    $container: ElementFinder;
    $input: ElementFinder;
    buttons: {
        $today: ElementFinder;
        $yesterday: ElementFinder;
        $same: ElementFinder;
        $date: ElementFinder;
        $time: ElementFinder;
    };
    calendar: {
        $today: ElementFinder;
        waitAnimation: () => Promise<void>;
    };
    clock: {
        "$01:00": ElementFinder;
        waitAnimation: () => Promise<void>;
    };
}
export interface BooleanWidgetPO {
    $container: ElementFinder;
    $true: ElementFinder;
    $false: ElementFinder;
    $undefined: ElementFinder;
    $active: ElementFinder;
    $nonactive: ElementFinder;
}
export interface EnumWidgetPOI {
    $container: ElementFinder;
    openEnums: () => Promise<void>;
    $enumContainer: ElementFinder;
    $$enums: ElementArrayFinder;
    $input: ElementFinder;
}
interface FormProps {
    schema?: JSONSchema7;
    uiSchema?: any;
    formData?: any;
    uiSchemaContext?: any;
    isAdmin?: boolean;
    isEdit?: boolean;
    id?: string;
    localFormData?: boolean | string;
}
export declare class Form {
    props: FormProps;
    contextId: number;
    constructor(params?: FormProps);
    initialize(beforeInit?: (form: Form) => Promise<void>, contextId?: number): Promise<void>;
    e(path: string): Promise<any>;
    setState(state: any): Promise<any>;
    getState(): Promise<any>;
    submit(): Promise<void>;
    startSubmit(): Promise<void>;
    submitOnlySchemaValidations(): Promise<void>;
    waitUntilBlockingLoaderHides(timeout?: number): import("selenium-webdriver").promise.Promise<unknown>;
    getSubmittedData(): Promise<any>;
    getChangedData(): Promise<any>;
    getPropsData(): Promise<any>;
    $form: ElementFinder;
    $locate(path: string): ElementFinder;
    $locateButton(path: string, selector: string): ElementFinder;
    $locateAddition(path: string, selector: string): ElementFinder;
    getMockStr: (path: string, query: any) => string;
    setMockResponse(path: string, query?: any): Promise<Mock>;
    createMockResponseQueue(path: string, query?: any): Promise<{
        create: () => Promise<{
            resolve: (response: any, raw?: boolean | undefined) => import("selenium-webdriver").promise.Promise<unknown>;
            reject: (response: any, raw?: boolean | undefined) => import("selenium-webdriver").promise.Promise<unknown>;
        }>;
        remove: () => import("selenium-webdriver").promise.Promise<unknown>;
    }>;
    createValidatorPO: (type: "error" | "warning") => {
        $$all: ElementArrayFinder;
        $panel: ElementFinder;
    };
    errors: {
        $$all: ElementArrayFinder;
        $panel: ElementFinder;
    };
    warnings: {
        $$all: ElementArrayFinder;
        $panel: ElementFinder;
    };
    failedJobs: {
        $container: ElementFinder;
    };
    $acknowledgeWarnings: ElementFinder;
    isBlocked: () => import("selenium-webdriver").promise.Promise<boolean>;
    mockImageUpload: (lajiFormLocator: string) => Promise<{
        resolve: () => Promise<void>;
        remove: () => Promise<void>;
    }>;
    getBooleanWidget(str: string): BooleanWidgetPO;
    $getInputWidget(str: string): ElementFinder;
    $getTextareaWidget(str: string): ElementFinder;
    $getEnumWidget(str: string): EnumWidgetPOI;
    getDateWidget(str: string): DateWidgetPO;
    $$getFieldErrors(str: string): ElementArrayFinder;
    _getImageArrayField: (form: Form) => (lajiFormLocator: string) => ImageArrayFieldPOI;
    getImageArrayField: (lajiFormLocator: string) => ImageArrayFieldPOI;
    _getTaxonAutosuggestWidget: (form: Form) => (lajiFormLocator: string) => TaxonAutosuggestWidgetPOI;
    getTaxonAutosuggestWidget: (lajiFormLocator: string) => TaxonAutosuggestWidgetPOI;
    getScopeField: (lajiFormLocator: string) => {
        $button: ElementFinder;
        $$listItems: ElementArrayFinder;
    };
    getUnitListShorthandArrayField: (lajiFormLocator: string) => UnitListShorthandArrayFieldPOI;
}
export declare const isDisplayed: ($elem: ElementFinder) => Promise<boolean>;
export interface ImageArrayFieldPOI {
    $container: ElementFinder;
    $$imgs: ElementArrayFinder;
    $$imgInteractives: ElementArrayFinder;
    $$imgRemoves: ElementArrayFinder;
    $imgRemoveConfirmButton: (id: string) => ElementFinder;
    $dropzone: ElementFinder;
    $modal: ElementFinder;
    $modalClose: ElementFinder;
}
export interface TaxonAutosuggestWidgetPOI {
    $input: ElementFinder;
    $suggestionsContainer: ElementFinder;
    $$suggestions: ElementArrayFinder;
    waitForSuggestionsToLoad: () => Promise<void>;
    waitForGlyph: () => Promise<void>;
    isSuggested: () => Promise<boolean>;
    isNonsuggested: () => Promise<boolean>;
    $powerUserButton: ElementFinder;
    powerUserButtonIsActive: () => Promise<boolean>;
    waitForPopoverToHide: () => Promise<void>;
}
export interface UnitListShorthandArrayFieldPOI {
    $button: ElementFinder;
    modal: {
        $input: ElementFinder;
        $addButton: ElementFinder;
    };
}
export declare class NamedPlaceChooserPO {
    select: EnumWidgetPOI;
    mapPopup: {
        $container: ElementFinder;
        $useBtn: ElementFinder;
    };
    $alert: ElementFinder;
    $close: ElementFinder;
}
export declare function createForm(props?: FormProps, beforeInit?: (form: Form) => Promise<void>): Promise<Form>;
export declare function waitUntilBlockingLoaderHides(timeout?: number): import("selenium-webdriver").promise.Promise<unknown>;
export declare function putForeignMarkerToMap(): Promise<void>;
export declare function removeUnit(gatheringIdx: number, unitIdx: number): Promise<void>;
export declare const updateValue: ($input: ElementFinder, value: string, blur?: boolean) => Promise<void>;
export declare const mockImageMetadata: {
    id: string;
    capturerVerbatim: string[];
    intellectualOwner: string;
    intellectualRights: string;
    fullURL: string;
    largeURL: string;
    squareThumbnailURL: string;
    thumbnailURL: string;
    originalURL: string;
    uploadedBy: string;
    "@context": string;
};
export declare const filterUUIDs: (any: any) => any;
export declare const maybeJSONPointerToLocator: (pointer: string) => string;
export {};
