"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("Trip report (JX.519) autosuggestions", () => {
    let form;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "JX.519" });
        yield test_utils_1.putForeignMarkerToMap();
    }));
    describe("observers", () => {
        it("when two unknown and 1st is removed, the autosuggest input value updated to the value of the 1st", () => __awaiter(void 0, void 0, void 0, function* () {
            // TableField has problems, so ids here are erratic but correct
            const $deleteObserver = form.$locateButton("gatheringEvent.0", "delete");
            const $addObserver = form.$locateButton("gatheringEvent", "add");
            const observer1$ = form.$getInputWidget("gatheringEvent.0.leg");
            const observer2$ = form.$getInputWidget("gatheringEvent.1.leg");
            yield $deleteObserver.click();
            yield $addObserver.click();
            yield $addObserver.click();
            yield observer1$.sendKeys("matti");
            yield observer2$.sendKeys("teppo");
            yield $deleteObserver.click();
            expect(yield observer1$.getAttribute("value")).toBe("teppo");
        }));
    });
    describe("taxon census", () => {
        let censusAutosuggest;
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            const scopeField = form.getScopeField("gatherings.0");
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(scopeField.$button), 1000, "gatherings additionals button didn't show up");
            yield scopeField.$button.click();
            yield scopeField.$$listItems.last().click();
            censusAutosuggest = form.getTaxonAutosuggestWidget("gatherings.0.taxonCensus.0.censusTaxonID");
        }));
        it("selects on click", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(form.$locateButton("gatherings.0.taxonCensus", "add")), 1000, "taxon census didn't show up");
            yield form.$locateButton("gatherings.0.taxonCensus", "add").click();
            yield censusAutosuggest.$input.sendKeys("aves");
            yield censusAutosuggest.waitForSuggestionsToLoad();
            yield censusAutosuggest.$$suggestions.first().click();
            yield censusAutosuggest.waitForGlyph();
            expect(yield censusAutosuggest.isSuggested()).toBe(true, "wasn't displayed as suggested");
            expect(yield censusAutosuggest.$input.getAttribute("value")).toBe("Aves", "Input value wasn't the clicked value");
        }));
    });
    describe("unit taxon", () => {
        let taxonAutosuggest;
        let $addUnit;
        const moveMouseAway = (locator) => protractor_1.browser.actions()
            .mouseMove(form.$locate(locator).getWebElement(), { x: -100, y: -100 })
            .perform();
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            taxonAutosuggest = form.getTaxonAutosuggestWidget("gatherings.0.units.0.identifications.0.taxon");
            $addUnit = form.$locateButton("gatherings.0.units", "add");
        }));
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            if (yield form.$locate("gatherings.0.units.0.identifications.0.taxon").isPresent()) {
                yield moveMouseAway("gatherings.0.units.0.identifications.0.taxon");
            }
        }));
        it("clicking any match selects it", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kett");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$$suggestions.first().click();
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and pressing enter after waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.ENTER);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and pressing enter without waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.ENTER);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and pressing tab after waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and pressing tab without waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing non exact match and blurring after waiting for suggestion list shows warning sign", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kett");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isNonsuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kett");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing non exact match and blurring without waiting for suggestion list shows warning sign", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kett");
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isNonsuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kett");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and blurring after waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.waitForSuggestionsToLoad();
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match and blurring before waiting for suggestion list selects exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            yield taxonAutosuggest.$input.sendKeys("kettu");
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
            yield taxonAutosuggest.waitForGlyph();
            expect(yield taxonAutosuggest.isSuggested()).toBe(true);
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe("kettu");
            expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
            yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
            yield protractor_1.browser.sleep(100);
            yield test_utils_1.removeUnit(0, 0);
            yield $addUnit.click();
        }));
        it("typing exact match with sp suffix keeps sp suffix in value but select the exact match", () => __awaiter(void 0, void 0, void 0, function* () {
            for (const suffix of ["sp", "spp", "sp.", "spp."]) {
                yield taxonAutosuggest.$input.sendKeys(`parus ${suffix}`);
                yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.TAB);
                yield taxonAutosuggest.waitForGlyph();
                expect(yield taxonAutosuggest.isSuggested()).toBe(true);
                expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe(`parus ${suffix}`);
                expect(yield test_utils_1.getFocusedElement().getAttribute("id")).not.toBe(yield taxonAutosuggest.$input.getAttribute("id"), "next elem wasn't focused");
                yield test_utils_1.getFocusedElement().sendKeys(protractor_1.protractor.Key.TAB); // Tab to get rid of toggler tooltip obscuring elements
                yield protractor_1.browser.sleep(100);
                yield test_utils_1.removeUnit(0, 0);
                yield $addUnit.click();
            }
        }));
        describe("power user", () => {
            it("is shown power taxon field", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield taxonAutosuggest.$powerUserButton.isDisplayed()).toBe(true);
            }));
            it("click toggles power user mode on/off", () => __awaiter(void 0, void 0, void 0, function* () {
                yield taxonAutosuggest.$powerUserButton.click();
                expect(yield taxonAutosuggest.powerUserButtonIsActive()).toBe(true);
                yield taxonAutosuggest.$powerUserButton.click();
                expect(yield taxonAutosuggest.powerUserButtonIsActive()).toBe(false);
            }));
            it("clicking any match after waiting for suggestion list works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield taxonAutosuggest.$powerUserButton.click();
                yield taxonAutosuggest.$input.sendKeys("kettu");
                yield taxonAutosuggest.waitForSuggestionsToLoad();
                yield taxonAutosuggest.$$suggestions.last().click();
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(form.$locate("gatherings.0.units.1")), 5000, "auto added unit didn't appear");
                expect(yield form.$locate("gatherings.0.units.0").getTagName()).toBe("tr", "entered unit wasn't deactivated");
                const enteredUnitTaxon = yield form.$locate("gatherings.0.units.0").$$("td").first().getText();
                expect(enteredUnitTaxon).toBe("ravunsyöjäkettu");
            }));
            it("entering second unit auto adds", () => __awaiter(void 0, void 0, void 0, function* () {
                const _taxonAutosuggest = form.getTaxonAutosuggestWidget("gatherings.0.units.1.identifications.0.taxon");
                yield _taxonAutosuggest.$input.sendKeys("kettu");
                yield _taxonAutosuggest.waitForSuggestionsToLoad();
                yield _taxonAutosuggest.$$suggestions.last().click();
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(form.$locate("gatherings.0.units.2")), 5000, "auto added unit didn't appear");
                expect(yield form.$locate("gatherings.0.units.1").getTagName()).toBe("tr", "entered unit wasn't deactivated");
                const enteredUnitTaxon = yield form.$locate("gatherings.0.units.1").$$("td").first().getText();
                expect(enteredUnitTaxon).toBe("ravunsyöjäkettu");
                yield test_utils_1.removeUnit(0, 0);
                yield test_utils_1.removeUnit(0, 0);
                yield test_utils_1.removeUnit(0, 0);
                yield $addUnit.click();
                yield taxonAutosuggest.$powerUserButton.click();
            }));
            it("entering exact match adds new unit automatically", () => __awaiter(void 0, void 0, void 0, function* () {
                yield taxonAutosuggest.$powerUserButton.click();
                yield taxonAutosuggest.$input.sendKeys("susi");
                yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.ENTER);
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(form.$locate("gatherings.0.units.1")), 5000, "New unit didn't show up");
                expect(yield form.$locate("gatherings.0.units.0").getTagName()).toBe("tr");
                expect(yield form.$locate("gatherings.0.units.1").getTagName()).toBe("div");
                yield test_utils_1.removeUnit(0, 1);
                yield taxonAutosuggest.$powerUserButton.click();
                expect(yield taxonAutosuggest.powerUserButtonIsActive()).toBe(false);
                yield test_utils_1.removeUnit(0, 0);
                yield $addUnit.click();
            }));
        });
        it("works when autocomplete response has autocompleteSelectedName", () => __awaiter(void 0, void 0, void 0, function* () {
            const mock = yield form.setMockResponse("/autocomplete/taxon");
            yield taxonAutosuggest.$input.sendKeys("kuusi");
            const autocompleteSelectedName = "foobar";
            yield mock.resolve([{
                    "key": "MX.37812",
                    "value": "kuusi",
                    autocompleteSelectedName,
                    "payload": {
                        "matchingName": "kuusi",
                        "informalTaxonGroups": [
                            {
                                "id": "MVL.343",
                                "name": "Putkilokasvit"
                            }
                        ],
                        "scientificName": "Picea abies",
                        "scientificNameAuthorship": "(L.) H. Karst.",
                        "taxonRankId": "MX.species",
                        "matchType": "exactMatches",
                        "cursiveName": true,
                        "finnish": true,
                        "species": true,
                        "nameType": "MX.obsoleteVernacularName",
                        "vernacularName": "metsäkuusi"
                    }
                }]);
            // idk why needed...
            yield protractor_1.browser.sleep(100);
            yield taxonAutosuggest.$input.sendKeys(protractor_1.protractor.Key.ENTER);
            yield mock.remove();
            expect(yield taxonAutosuggest.$input.getAttribute("value")).toBe(autocompleteSelectedName);
            expect((yield form.getChangedData()).gatherings[0].units[0].identifications[0].taxon).toBe(autocompleteSelectedName);
        }));
    });
});
