"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("WBC (MHL.3)", () => {
    let form;
    it("navigate to form", () => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.3", localFormData: true, isAdmin: true });
    }));
    it("taxon census has aves", () => __awaiter(void 0, void 0, void 0, function* () {
        const formData = yield form.getPropsData();
        expect(formData.gatherings[0].taxonCensus[0].censusTaxonID).toBe("MX.37580");
    }));
    it("taxon census has correct values when yes is selected", () => __awaiter(void 0, void 0, void 0, function* () {
        const widget = form.getBooleanWidget("gatherings.0.taxonCensus");
        yield widget.$true.click();
        const formData = yield form.getChangedData();
        expect(formData.gatherings[0].taxonCensus.length).toBe(2);
        expect(formData.gatherings[0].taxonCensus[0].censusTaxonID).toBe("MX.37580");
        expect(formData.gatherings[0].taxonCensus[1].censusTaxonID).toBe("MX.37612");
    }));
    it("taxon census has correct values when no is selected", () => __awaiter(void 0, void 0, void 0, function* () {
        const widget = form.getBooleanWidget("gatherings.0.taxonCensus");
        yield widget.$false.click();
        const formData = yield form.getChangedData();
        expect(formData.gatherings[0].taxonCensus.length).toBe(1);
        expect(formData.gatherings[0].taxonCensus[0].censusTaxonID).toBe("MX.37580");
    }));
    it("added existing user is shown as suggested", () => __awaiter(void 0, void 0, void 0, function* () {
        // ID for button is broken, should be gatheringEvent.leg?
        yield form.$locateButton("gatheringEvent", "add").click();
        const $field = form.$locate("gatheringEvent.1.leg");
        const $input = form.$getInputWidget("gatheringEvent.1.leg");
        yield $input.sendKeys("unit");
        const $suggestionList = $field.$(".rw-list");
        const $$suggestions = $field.$$(".rw-list-option");
        yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf($suggestionList), 5000, "Suggestion list timeout");
        yield $$suggestions.first().click();
        yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.invisibilityOf($field.$(".react-spinner")), 5000, "Field input spinner didn't leave");
        expect(yield form.$locate("gatheringEvent.1.leg").$(".glyphicon-user").isDisplayed()).toBe(true);
    }));
    describe("namedPlaceNotes", () => {
        it("renders", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$locate("gatherings.0.namedPlaceNotes").isDisplayed()).toBe(true);
        }));
        it("is textarea", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$getTextareaWidget("gatherings.0.namedPlaceNotes").isDisplayed()).toBe(true);
        }));
        it("updates", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(form.$getTextareaWidget("gatherings.0.namedPlaceNotes"), "test");
            expect(yield form.$getTextareaWidget("gatherings.0.namedPlaceNotes").getAttribute("value")).toBe("test");
            expect((yield form.getChangedData()).gatheringEvent.namedPlaceNotes).toBe("test");
        }));
        it("doesn't spill into gatherings", () => __awaiter(void 0, void 0, void 0, function* () {
            expect((yield form.getChangedData()).gatherings[0]).not.toContain("namedPlaceNotes");
        }));
    });
    it("added existing user ID is shown", () => __awaiter(void 0, void 0, void 0, function* () {
        const $input = form.$getInputWidget("gatheringEvent.1.leg");
        expect(yield $input.getAttribute("value")).toMatch(/Unit.+ \(MA\.\d+\)/);
    }));
    it("entering descriptionBiotopeF doesn't clear observerCount", () => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.3", localFormData: true, isAdmin: true });
        const $descriptionBiotopeF = form.$locate("gatherings.0.gatheringFact.descriptionBiotopeF");
        const $otherBiotopeFCheckbox = $descriptionBiotopeF.$$(".checkbox label input").get(2);
        yield $otherBiotopeFCheckbox.click();
        yield $otherBiotopeFCheckbox.click();
        yield $otherBiotopeFCheckbox.click();
        //await $descriptionBiotopeF.$("input.form-control").sendKeys("test");
        const formData = yield form.getChangedData();
        expect(formData.gatherings[0].gatheringFact.observerCount).toBe(1);
    }));
});
